#version 330 core

struct SimpleDirectionalLight {
	vec4 vColor;
	vec3 vDirection;
	float fAmbientIntensity;
};

struct Material {
	vec4 emission;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
};

uniform SimpleDirectionalLight sunLight;
uniform vec3 materialColor;
uniform sampler2D textureSample;
uniform Material material;
uniform bool drawShadow;
smooth in vec3 vNormal;
in vec2 uv;
in vec4 eye;

uniform sampler2DShadow shadowMap;
in vec4 smCoords;

out vec4 outputColor;
out vec4 outputNormal;

const highp vec3 W = vec3(0.2125f, 0.7154f, 0.0721f);

float mapIntensity(float intensity) {
	if (intensity > 0.99) return 1.0;
	else if (intensity > 0.9) return 0.9;
	else if (intensity > 0.8) return 0.8;
	else if (intensity > 0.7) return 0.7;
	else if (intensity > 0.6) return 0.6;
	else if (intensity > 0.5) return 0.5;
	else if (intensity > 0.4) return 0.4;
	else if (intensity > 0.3) return 0.3;
	else if (intensity > 0.2) return 0.2;
	else if (intensity > 0.1) return 0.1;
	else return 0.f;
}

void main(){
	vec3 N = normalize(vNormal);
	vec3 L = normalize(-sunLight.vDirection);
	vec4 surfaceColor = texture(textureSample, uv);

	float shadowMult = 1.0f;
	if (drawShadow) {
		shadowMult = texture(shadowMap, (smCoords.xyz/smCoords.w));
	}

	//ambient
	vec3 ambient = sunLight.fAmbientIntensity * 0.3 * surfaceColor.rgb * sunLight.vColor.rgb;

	//diffuse
	float diffuseCoefficient = max(0.0, dot(N, L));
	vec3 diffuse = sunLight.fAmbientIntensity * diffuseCoefficient * surfaceColor.rgb * sunLight.vColor.rgb;

	//specular
	float specularCoefficient = 0.0;
	if (diffuseCoefficient > 0.0f) {
		specularCoefficient = pow(max(0.0, dot(normalize(eye.xyz), reflect(-L, N))), 100.f);
	}
	vec3 specular = sunLight.fAmbientIntensity * specularCoefficient * surfaceColor.rgb * sunLight.vColor.rgb;

	vec3 color = diffuse + specular;
	float shadowIntens = mapIntensity(shadowMult);
	float intensity = mapIntensity(dot(color, W));

	outputColor = vec4(ambient + shadowIntens * intensity * color, surfaceColor.a);
	outputNormal = vec4(vec3(dot(N, W)), 1.f);

    if(outputColor.a == 0.0){
            discard;
    }
}
